#ifndef MQBENCH_H
#define MQBENCH_H

#include <stdio.h>
#include "osspsfc.h"

#if defined(MSMQ_VER) || defined(FMQC_VER)
# include "msmqver.h"
#elif defined(MQSR_VER)
# include "mqsrver.h"
#else
# error No Platform defined!!
#endif

#ifdef __cplusplus 
  extern "C" {
#endif

#define DEF2STR(dw) case (dw): psz = #dw; break;
#define SLEEP_FACTOR 250

typedef void* PTRANSOBJ;
typedef enum { SEND, RECEIVE } OPERATION;
typedef enum { EXPRESS, RECOVERABLE} MSGTYPE;
typedef enum { APP_Q, SENDSYNC_Q, RECVSYNC_Q } QTYPE;

typedef enum { NO_SYNC, SYNC_SEND_THEN_RECEIVE, SYNC_RECEIVE_WHILE_SENDING } SYNCMODE;
typedef enum { BEGINSYNC, NORMALSYNC } SYNCOPERATION;
static char* SYNCOPERTEXT[] = {"BEGINSYNC ", "NORMALSYNC"};

#define SYNCOPERTEXTSIZE  11
#define SYNC_TIMEOUT      60000   /* set the sync timeout to 60 seconds */
#define SYNC_MSG_LIFETIME 60000   /* set the lifetime for a sync message to 60 seconds */

typedef struct {
  HANDLE hSendSyncQ;
  HANDLE hRecvSyncQ;
  char   pSenderId[MAX_PATH];
  char   pReceiverId[MAX_PATH];
} SYNCDATA;

typedef struct {
	UCHAR MsgType;
	OPERATION Operation;
	XACTTYPE_I XactType;
  SYNCMODE    SyncMode;
	ULONG ulMsgNumber;
	ULONG ulTransactionNum;
	ULONG ulThreadNum;
	ULONG ulMsgSize;
	BOOL fWaitForReceivedMsg;
	BOOL fOperationSpecified;
  BOOL fClearQueue;
  DEFS_I I;  /* defines necessary for the particular interface */
} PARAMS;

extern EVENTOBJ_OS g_hWaitToStart;
extern EVENTOBJ_OS g_hStart;
extern EVENTOBJ_OS g_hEnd;
extern EVENTOBJ_OS g_hSync1;
extern EVENTOBJ_OS g_hSync2;
extern LONG g_ThreadCounter;
extern PARAMS g_Params;
extern int g_fTimerStarted;

/* prototypes for common routines */
void ErrorExit (char *pszErrorMsg, HRESULT hr);
void InitParams(void);
int IsValidParam (int Cur, int ParamNum, char *pszParams[]);
void GetMsgType(char chMsgType);
void GetMessageParams(int Cur, int ParamNum, char *pszParams[]);
int ParseSendParams (int Cur, int ParamNum, char *pszParams[]);
int ParseReceiveParams (int Cur, int ParamNum, char *pszParams[]);
int ParsePathParams(int Cur, int ParamNum, char *pszParams[]);
int ParseThreadParams(int Cur, int ParamNum, char *pszParams[]);
int ParseClearParams(int Cur, int ParamNum, char *pszParams[]);
void GetSyncType(char chSyncType);
void GetSyncParams(int Cur, int ParamNum, char *pszParams[]);
int ParseSyncParams(int Cur, int ParamNum, char *pszParams[]);
int ParseWaitParams(int Cur);
void ParseParams (int ParamNum, char *pszParams[]);
char* OPER2STR(DWORD dwOper);
char* MSGTYPE2STR(DWORD dwMsgType);
char* SYNCMODE2STR(DWORD dwSyncMode);
void ResultOutput(void);
void CreateEvents(void);
DWORD APIENTRY TestThread(PVOID);
void RunTest(void);
void BeginSync(HANDLE hQueue, SYNCDATA* pSyncData);
void EndSync(HANDLE hQueue, SYNCDATA* pSyncData);
void PerfomBeginSendSync(HANDLE hQueue, SYNCDATA* pSyncData);
void PerfomBeginRecvSync(HANDLE hQueue, SYNCDATA* pSyncData);
void PerfomEndSendSync(HANDLE hQueue, SYNCDATA* pSyncData);
void PerfomEndRecvSync(HANDLE hQueue, SYNCDATA* pSyncData);
void CloseEvents(void);

/* prototypes defined per platform */
void Usage_I(void);
void InitParams_I(void);
int ParseTransactionParams_I(int Cur, int ParamNum, char *pszParams[]);
void ParsePathParams_I(char* pszPathname,QTYPE qType);
int IsValidPathname_I(void);
char* XACTTYPE2STR_I(DWORD dwXactType);
void InitTest_I(void);
void SetMessageProps_I(void);
HANDLE GetQueueHandle_I(void);
PTRANSOBJ GetTransactionPointer_I(void);
void SendMsg_I(PTRANSOBJ pXact,HANDLE hQueue);
void RecvMsg_I(PTRANSOBJ pXact, HANDLE hQueue);
void CommitTrans_I(PTRANSOBJ pXact, HANDLE hQueue);
void CloseQueue_I(HANDLE* phQueue);
void PrintQueuename_I(QTYPE qType);
DWORD ClearQueue_I(HANDLE hQueue);
void OpenSyncQueues_I(HANDLE hQueue, SYNCDATA* pSyncData);
void CloseSyncQueues_I(HANDLE hQueue, SYNCDATA* pSyncData);
void GetSyncMessage_I(HANDLE hQueue, SYNCDATA* pSyncData,char* pMsgId, char* pCorrId, 
                      SYNCOPERATION SyncOper);
void PutSyncMessage_I(HANDLE hQueue, SYNCDATA* pSyncData,char* pMsgId, char* pCorrId, 
                      SYNCOPERATION SyncOper);
void CleanupTest_I(void);

#ifdef __cplusplus 
  }
#endif

#endif
